#!/bin/sh
#
#  copy libxpg4 and patch it for Netscape Communicator 4.0x
#
#  from [FreeBSD-tech-jp 1823] and [FreeBSD-tech-jp 1896]
#
DIR1=/usr/src/lib/libc/locale

cd $DIR1
cp -p ansi.c ansi-ns.c
chmod +w ansi-ns.c

#
# apply [FreeBSD-tech-jp 1823] patch (first)
#

patch -N ansi-ns.c <<END_OF_DIFF
--- ansi.c.orig	Thu Apr 18 16:01:46 1996
+++ ansi.c	Mon Oct 26 01:55:26 1998
@@ -105,9 +105,18 @@
 	char const *e;
 	int cnt = 0;
 
-	if (!pwcs || !s)
+	if (!s)
 		return (-1);
 
+	if (!pwcs) {
+		while (*s) {
+			if (sgetrune(s, MB_LEN_MAX, &e) == _INVALID_RUNE)
+				return (-1);
+			s = e;
+			++cnt;
+		}
+		return (cnt);
+	}
 	while (n-- > 0) {
 		*pwcs = sgetrune(s, MB_LEN_MAX, &e);
 		if (*pwcs == _INVALID_RUNE)
@@ -139,8 +148,10 @@
 		}
 		if (!sputrune(*pwcs++, s, n, &e))
 			return (-1);		/* encoding error */
-		if (!e)			/* too long */
+		if (!e) {			/* too long */
+			*s = 0;
 			return (cnt);
+		}
 		cnt += e - s;
 		n -= e - s;
 		s = e;
END_OF_DIFF

#
# apply [FreeBSD-tech-jp 1896] patch.
#

patch -N ansi-ns.c <<END_OF_DIFF
--- ansi.c.pat2	Fri Nov  6 05:33:00 1998
+++ ansi.c	Fri Nov  6 05:53:03 1998
@@ -141,6 +141,12 @@
 	if (!pwcs || !s)
 		return (-1);
 
+if( (pwcs[0]&0xffff0000) || ((!(pwcs[0]&0x8000))&&(pwcs[0]&0xff00)) ){
+  int len;
+  strncpy(s, (char *)pwcs, n);
+  len = strlen((char *)pwcs);
+  return (len < n) ? len : n;
+}
 	while (n > 0) {
 		if (*pwcs == 0) {
 			*s = 0;
END_OF_DIFF

rm *.orig

#
# make libxpg4-ns directory
#

DIR2=/usr/src/lib/
cd $DIR2
rm -rf libxpg4-ns
cp -Rp libxpg4 libxpg4-ns
cd libxpg4-ns


patch -N << END_OF_DIFF
--- Makefile.orig	Wed Feb 18 14:33:31 1998
+++ Makefile	Mon Nov  9 09:43:04 1998
@@ -1,6 +1,6 @@
-LIB=    xpg4
+LIB=    xpg4-ns
 SRCS=   setlocale.c setrunelocale.c euc.c mskanji.c utf2.c runetype.c \\
-	tolower.c toupper.c
+	tolower.c toupper.c ansi-ns.c
 CFLAGS+= -Wall -DXPG4 -I\${.CURDIR}/../libc/locale
 .PATH: \${.CURDIR}/../libc/locale
END_OF_DIFF

make clean
make
make install

# 
# end of file
#
